import React from 'react';
import { hydrateRoot } from 'react-dom';
import { BrowserRouter } from 'react-router-dom';
import ProjectClient from 'pawProjectClient';
import ProjectRoutes from 'pawProjectRoutes';
import { PartialRouteObject } from 'react-router';
import { RoutesHandler } from '../router/handler';
import { AbstractPlugin } from '../common/abstract-plugin';
import { App } from '../components/app';
import { Route } from '../../index';
import { PreloaderProvider } from '../components/data';
import { suspensify } from '../components/suspensify';

class ClientHandler extends AbstractPlugin {
  #routes: Route[] = [];

  #compiledRoutes: PartialRouteObject[] = [];

  #projectRoutes: ProjectRoutes;

  readonly #url: URL;

  readonly #userAgent: string;

  constructor() {
    super();
    const projectClientInstance: AbstractPlugin = new ProjectClient();
    this.addPlugin(projectClientInstance);
    this.#projectRoutes = new ProjectRoutes();
    /**
     * @todo: add polyfill.io with URL requirement
     */
    this.#url = new window.URL(window.location.href);
    this.#userAgent = window?.navigator?.userAgent ?? '';
  }

  async initRoutes() {
    const languages = Array.from(navigator?.languages ?? []);
    const language = navigator.language ?? (
      languages.length ? languages[0] : ''
    );
    const returnValue: Route [] | Promise<Route []> = this
      .#projectRoutes
      ?.getRoutes?.({
        languages,
        language,
        url: this.#url,
        userAgent: this.#userAgent,
      });

    this.#routes = await returnValue;
    const routeHandler = new RoutesHandler();
    this.#compiledRoutes = routeHandler.compileRoutes(this.#routes);
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars,class-methods-use-this
  addPlugin(plugin: AbstractPlugin): any {
    // Do nothing with the plugin as of now
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars,class-methods-use-this
  async run() {
    await this.initRoutes();
    const assets = window?.assetsManifest ?? [];
    const preloadedData = window?.pawPreloadedData ?? {};
    const suspensifiedData: any = {};
    Object.keys(preloadedData).forEach((preloadedKey) => {
      suspensifiedData[preloadedKey] = suspensify(preloadedData[preloadedKey]);
    });
    // eslint-disable-next-line no-console
    // @ts-ignore
    hydrateRoot(document, (
      <PreloaderProvider context={suspensifiedData}>
        <BrowserRouter>
          <App
            assets={assets}
            routes={this.#compiledRoutes}
          />
        </BrowserRouter>
      </PreloaderProvider>
    ));
  }
}

export { ClientHandler };
