import React, {
  Suspense, PropsWithChildren,
} from 'react';
import { DataProvider, PreloaderData } from './data';

type HtmlParams = {
  title?: string,
  assets: string [],
  children: React.ReactNode,
};

const extractAssets = (
  assets: string[],
  endingWith: string,
) => assets.filter((asset: string) => asset.endsWith(endingWith));

const Html = (props: PropsWithChildren<HtmlParams>) => {
  const {
    title, assets, children,
  } = props;
  const jsAssets = extractAssets(assets, '.js');
  return (
    <html lang="en">
      <head>
        <meta charSet="utf-8" />
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        <title>{title ?? 'ReactPWA'}</title>
      </head>
      <body>
        {children}
        {
          jsAssets.map((asset: string) => (
            <script key={asset} defer async src={asset} />
          ))
        }
        <script
          // eslint-disable-next-line react/jsx-props-no-multi-spaces
          dangerouslySetInnerHTML={{
            __html: `window.assetsManifest = ${JSON.stringify(assets)};`,
          }}
        />
        <DataProvider>
          <Suspense fallback={<></>}>
            <PreloaderData />
          </Suspense>
        </DataProvider>
      </body>
    </html>
  );
};
export { Html };
