export const suspensify = (promise: any) => {
  let status: 'success' | 'error' | 'pending' = 'pending';
  let response: any;
  let error: Error;
  let suspender: Promise<any>;
  if (!promise?.then && !(promise instanceof Promise)) {
    status = 'success';
    response = typeof promise === 'function' ? promise() : promise;
  } else if (promise instanceof Promise) {
    suspender = promise.then(
      (res) => {
        status = 'success';
        response = res;
      },
      (err: Error) => {
        status = 'error';
        error = err;
      },
    );
  }
  return {
    read() {
      if (status === 'pending' && suspender) {
        // eslint-disable-next-line @typescript-eslint/no-throw-literal
        throw suspender;
      }
      if (status === 'error') {
        throw error;
      }
      if (status === 'success') {
        return response;
      }
      throw new Error(`Invalid status of suspensify promise "${status}". Valid status should be in (pending, success, error)`);
    },
  };
};
