import React, { Suspense } from 'react';
import { RouteObject } from 'react-router';
import { Route } from '../../index';
import { DataProvider } from '../components/data';

class RoutesHandler {
  #userRoutes: Route [] = [];

  #compiledRoutes: RouteObject [] = [];

  #defaultFallbackComponent: React.FC | React.ClassicComponent;

  constructor() {
    this.#defaultFallbackComponent = () => <>Loading...</>;
  }

  setUserRoutes(routes: Route []) {
    this.#userRoutes = routes;
  }

  getUserRoutes() {
    return this.#userRoutes;
  }

  compileAllRoutes() {
    this.#compiledRoutes = [];
    this.#compiledRoutes = this.compileRoutes(this.#userRoutes);
    return this.#compiledRoutes;
  }

  compileRoutes(routes: Route[]): RouteObject[] {
    const compiledRoutes: RouteObject [] = [];
    routes.forEach((userRoute: Route) => {
      const FallbackComponent = userRoute.loader ?? this.#defaultFallbackComponent;
      const Element = userRoute.component;
      const route: RouteObject = {
        children: this.compileRoutes(userRoute.routes ?? []),
        path: userRoute.path,
        caseSensitive: userRoute?.caseSensitive ?? false,
        element: (
          <DataProvider>
            {/* @ts-ignore */}
            <Suspense fallback={<FallbackComponent />}>
              {/* eslint-disable-next-line react/jsx-props-no-spreading */}
              <Element {...userRoute.props} />
            </Suspense>
          </DataProvider>
        ),
      };
      compiledRoutes.push(route);
    });
    return compiledRoutes;
  }
}

export { RoutesHandler };
