import path from 'path';
import fs from 'fs';
import { pawExistsSync } from '../util/common';
import packageJson from '../../package.json';

type StringObj = { [key: string]: string };

const libraryNodeModules = path.resolve(path.join(__dirname, '..', 'node_modules'));
const libraryHasNodeModules = libraryNodeModules ? fs.existsSync(libraryNodeModules) : false;

const root = process.cwd();

// Get files in the project root
const ProjectClientPath = pawExistsSync(path.join(root, 'src', 'client'));
const ProjectServerPath = pawExistsSync(path.join(root, 'src', 'server'));
const ProjectRoutesPath = pawExistsSync(path.join(root, 'src', 'routes'));

const resolveFirstToLibDependencies = (dependencies: StringObj): StringObj => {
  const resolvedDependencies: StringObj = {};
  Object.keys(dependencies).forEach((dependency: string) => {
    resolvedDependencies[dependency] = fs.existsSync(path.join(libraryNodeModules, dependency))
      ? path.resolve(path.join(libraryNodeModules, dependency))
      : path.resolve(path.join(root, 'node_modules', dependency));
  });
  return resolvedDependencies;
};

/**
 * Resolve dependencies in the library's node_modules as priority
 */
const dependenciesAlias = resolveFirstToLibDependencies(packageJson.dependencies);

// Get alternate file paths in shared folder
const srcPath = path.resolve(path.join(
  __dirname,
  '..',
  'src',
));
const EmptyPluginPath = path.resolve(
  path.join(
    srcPath,
    'common',
    'empty-plugin.ts',
  ),
);

export const Resolver = {
  resolve: {
    alias: {
      ...dependenciesAlias,
      pawProjectClient: ProjectClientPath || EmptyPluginPath,
      pawProjectServer: ProjectServerPath || EmptyPluginPath,
      pawProjectRoutes: ProjectRoutesPath || EmptyPluginPath,
    },
    extensions: ['.ts', '.tsx', '.js', '.jsx', '.json', '.wasm'],
  },
  resolveLoader: {
    modules: [
      'node_modules',
      ...(libraryHasNodeModules ? [libraryNodeModules] : []),
    ],
  },

};
